@@ -1,29 +1,40 @@
+fromStringToNumberArrowType <- function(val) {
+  ret <- 0
+  if (val == "arrow") {
+    ret <- 6
+  } else if (val == "TRUE") {
+    ret <- 15
+  }
+  return(ret)
+}
+
 #' GraphML converting function for miic graph
 #'
 #' @description Convert miic graph to [GraphML format](http://graphml.graphdrawing.org/).
-#' @param g The graph object returned by [miic][miic()].
+#' @param miic_obj A miic object. The object returned by the \code{\link{miic}} execution.
 #' @param file A string. Path to the output file containing file name without
 #' extension (.graphml will be appended).
 #' @param layout An optional data frame of 2 (or 3) columns containing the
 #' coordinate `x` and `y` for each node. The optional first column can contain
 #' node names. If node names is not given, the order of the input file will be
 #' assigned to the list of positions.
+#' @return None
 #' @export
 #' @useDynLib miic
 #' @md
 
-miic.write.network.cytoscape <- function(g, file, layout = NULL) {
+writeCytoscapeNetwork <- function(miic_obj, file, layout = NULL) {
   ##################################### NETWORK IN GRAPHML
   if (missing(file)) {
     stop("The file path is necessary")
   }
 
-  if (is.null(g$all.edges.summary)) {
+  if (is.null(miic_obj$summary)) {
     stop("The result of the miic execution is required")
   }
 
-  summary <- g$all.edges.summary
-  adj_matrix <- g$adj_matrix
+  summary <- miic_obj$summary
+  adj_matrix <- miic_obj$adj_matrix
 
   if (is.null(layout)) {
     line <- "<graphml>\n"
@@ -80,7 +91,7 @@ miic.write.network.cytoscape <- function(g, file, layout = NULL) {
     )
     line <- paste(
       line,
-      "\t<key id=\"log_confidence\" for=\"edge\" attr.name=\"log_confidence\" attr.type=\"double\"/>\n",
+      "\t<key id=\"info_shifted\" for=\"edge\" attr.name=\"info_shifted\" attr.type=\"double\"/>\n",
       sep = ""
     )
     line <- paste(
@@ -128,12 +139,12 @@ miic.write.network.cytoscape <- function(g, file, layout = NULL) {
 
     # cicle on edges
     for (index in indexes) {
-      if (!is.na(summary[index, "log_confidence"])) {
-        weigth <- summary[index, "log_confidence"]
+      if (!is.na(summary[index, "info_shifted"])) {
+        weigth <- summary[index, "info_shifted"]
       } else {
         weigth <- (summary[index, "partial_correlation"])
       }
-      if (summary[index, "infOrt"] == 1) {
+      if (summary[index, "ort_inferred"] == 1) {
         line <- paste(
           line,
           "\t\t<edge target=\"",
@@ -156,7 +167,7 @@ miic.write.network.cytoscape <- function(g, file, layout = NULL) {
           sep =
             ""
         )
-      } else if (summary[index, "infOrt"] == 2) {
+      } else if (summary[index, "ort_inferred"] == 2) {
         line <- paste(
           line,
           "\t\t<edge target=\"",
@@ -219,7 +230,7 @@ miic.write.network.cytoscape <- function(g, file, layout = NULL) {
           sep =
             ""
         )
-      } else if (summary[index, "infOrt"] == -2) {
+      } else if (summary[index, "ort_inferred"] == -2) {
         line <- paste(
           line,
           "\t\t<edge target=\"",
@@ -282,7 +293,7 @@ miic.write.network.cytoscape <- function(g, file, layout = NULL) {
           sep =
             ""
         )
-      } else if (summary[index, "infOrt"] == 6) {
+      } else if (summary[index, "ort_inferred"] == 6) {
         line <- paste(
           line,
           "\t\t<edge target=\"",
@@ -355,13 +366,13 @@ miic.write.network.cytoscape <- function(g, file, layout = NULL) {
         )
       }
 
-      if (!all(is.na(summary[, "log_confidence"]))) {
-        if (summary[index, "log_confidence"] <= 1) {
+      if (!all(is.na(summary[, "info_shifted"]))) {
+        if (summary[index, "info_shifted"] <= 1) {
           value <- 1
-        } else if (summary[index, "log_confidence"] >= 20) {
+        } else if (summary[index, "info_shifted"] >= 20) {
           value <- 8
         } else {
-          value <- summary[index, "log_confidence"] * 8 / 20
+          value <- summary[index, "info_shifted"] * 8 / 20
         }
       } else {
         value <- (abs(summary[index, "partial_correlation"]) + 1) * 4
@@ -386,7 +397,7 @@ miic.write.network.cytoscape <- function(g, file, layout = NULL) {
       )
       line <- paste(line,
         "\t\t\t<data key=\"nSamples\">",
-        summary[index, "Nxy_ui"],
+        summary[index, "n_xy_ai"],
         "</data>\n",
         sep = ""
       )
@@ -397,8 +408,8 @@ miic.write.network.cytoscape <- function(g, file, layout = NULL) {
         sep = ""
       )
       line <- paste(line,
-        "\t\t\t<data key=\"log_confidence\">",
-        summary[index, "log_confidence"],
+        "\t\t\t<data key=\"info_shifted\">",
+        summary[index, "info_shifted"],
         "</data>\n",
         sep = ""
       )
@@ -485,7 +496,7 @@ miic.write.network.cytoscape <- function(g, file, layout = NULL) {
     for (index in indexes) {
       sourceArrowNum <- 0
       targetArrowNum <- 0
-      if (summary[index, "infOrt"] == 1) {
+      if (summary[index, "ort_inferred"] == 1) {
         line <- paste(
           line,
           "\t\t<edge label=\"",
@@ -503,7 +514,7 @@ miic.write.network.cytoscape <- function(g, file, layout = NULL) {
           "\t\t\t<att name=\"edgeType\" type=\"integer\" value=\"1\"/>\n",
           sep = ""
         )
-      } else if (summary[index, "infOrt"] == 2) {
+      } else if (summary[index, "ort_inferred"] == 2) {
         if (is.na(summary[index, "partial_correlation"])) {
           value <- "arrow"
           varchar <- intToUtf8(187)
@@ -557,7 +568,7 @@ miic.write.network.cytoscape <- function(g, file, layout = NULL) {
         )
         sourceArrowNum <- 0
         targetArrowNum <- fromStringToNumberArrowType(value)
-      } else if (summary[index, "infOrt"] == -2) {
+      } else if (summary[index, "ort_inferred"] == -2) {
         if (is.na(summary[index, "partial_correlation"])) {
           value <- "arrow"
           varchar <- intToUtf8(187)
@@ -611,7 +622,7 @@ miic.write.network.cytoscape <- function(g, file, layout = NULL) {
         )
         sourceArrowNum <- 0
         targetArrowNum <- fromStringToNumberArrowType(value)
-      } else if (summary[index, "infOrt"] == 6) {
+      } else if (summary[index, "ort_inferred"] == 6) {
         if (is.na(summary[index, "partial_correlation"])) {
           value <- "arrow"
           varchar <- intToUtf8(187)
@@ -670,12 +681,12 @@ miic.write.network.cytoscape <- function(g, file, layout = NULL) {
         targetArrowNum <- fromStringToNumberArrowType(value)
       }
 
-      if (summary[index, "log_confidence"] <= 1) {
+      if (summary[index, "info_shifted"] <= 1) {
         value <- 1
-      } else if (summary[index, "log_confidence"] >= 20) {
+      } else if (summary[index, "info_shifted"] >= 20) {
         value <- 8
       } else {
-        value <- summary[index, "log_confidence"] * 8 / 20
+        value <- summary[index, "info_shifted"] * 8 / 20
       }
 
       line <- paste(
@@ -702,7 +713,7 @@ miic.write.network.cytoscape <- function(g, file, layout = NULL) {
       line <- paste(
         line,
         "\t\t\t<att name=\"nSamples\" type=\"integer\" value=\"",
-        summary[index, "Nxy_ai"],
+        summary[index, "n_xy_ai"],
         "\"/>\n",
         sep = ""
       )
@@ -715,8 +726,8 @@ miic.write.network.cytoscape <- function(g, file, layout = NULL) {
       )
       line <- paste(
         line,
-        "\t\t\t<att name=\"log_confidence\" type=\"double\" value=\"",
-        summary[index, "log_confidence"],
+        "\t\t\t<att name=\"info_shifted\" type=\"double\" value=\"",
+        summary[index, "info_shifted"],
         "\"/>\n",
         sep = ""
       )
