@@ -31,25 +31,11 @@ Environment::Environment(int n_samples, int n_nodes, vector<int> vec_numeric,
   for (int i = 0; i < n_nodes; ++i) {
     has_na[i] = std::any_of(data_numeric.row_begin(i), data_numeric.row_end(i),
         [](int value) { return value == -1; });
-  }
-  for (int i = 0; i < n_nodes; ++i) {
-    for (int j = 0; j < n_nodes; ++j) {
-      if ((!is_continuous[i] && levels[i] == n_samples) ||
-          (!is_continuous[j] && levels[j] == n_samples)) {
-        // If a node is discrete with as many levels as there are samples, its
-        // information with other nodes is null.
-        edges(i, j).status = 0;
-        edges(i, j).status_prev = 0;
-      } else {
-        // Initialise all other edges.
-        edges(i, j).status = 1;
-        edges(i, j).status_prev = 1;
-      }
-    }
-  }
-  for (int i = 0; i < n_nodes; ++i) {
+
+    edges(i, i).status_init = 0;
     edges(i, i).status = 0;
     edges(i, i).status_prev = 0;
+    edges(i, i).proba_head = -1;
   }
 }
 
@@ -57,10 +43,14 @@ void Environment::readBlackbox(const Grid2d<int>& node_list) {
   int n_pairs = node_list.n_rows();
   for (int i = 0; i < n_pairs; ++i) {
     const auto pair = node_list.getConstRow(i);
+    edges(pair[0], pair[1]).status_init = 0;
     edges(pair[0], pair[1]).status = 0;
     edges(pair[0], pair[1]).status_prev = 0;
+    edges(pair[0], pair[1]).proba_head = -1;
+    edges(pair[1], pair[0]).status_init = 0;
     edges(pair[1], pair[0]).status = 0;
     edges(pair[1], pair[0]).status_prev = 0;
+    edges(pair[1], pair[0]).proba_head = -1;
   }
 }
 
